#!/usr/bin/perl
#
################################################################################
#
# IPCop GUI page for Network UPS Tools (NUT)
#
# Copyright (C) 2014 Olaf (weizen_42) Westrik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
################################################################################
#
# For support post / read in http://www.ipcop-forum.de
#
# $Id: nutgui.cgi 546 2014-04-17 19:59:12Z weizen_42 $
#

# Add entry in menu
# MENUENTRY services 075 "NUT" "NUT"

use strict;

# enable only the following on debugging purpose
use warnings;
use CGI::Carp 'fatalsToBrowser';

use LWP::UserAgent;
use UPS::Nut;

require '/usr/lib/ipcop/general-functions.pl';
require '/usr/lib/ipcop/lang.pl';
require '/usr/lib/ipcop/header.pl';


my $version = 'v0.1.0';
my $addonname = 'nut';
my $onlineversion = '';
my $onlinelink = '';
my $timestamplastcheck = '/var/ipcop/addons/nut/lastcheck';
my $flagdonotcheck = '/var/ipcop/addons/nut/noversioncheck';
my $debug = 0;
my $errormessage = '';

my %checked = ();
my %selected=();
my %hashcgi=();
$hashcgi{'ACTION'} = '';
$hashcgi{'MODE'} = 'none';
$hashcgi{'SERVER'} = '';
$hashcgi{'UPSNAME'} = '';

my %hashsettings = ();
$hashsettings{'MODE'} = 'none';
$hashsettings{'SERVER'} = '';
$hashsettings{'UPSNAME'} = 'ups';
$hashsettings{'USERNAME'} = 'ups';
$hashsettings{'PASSWORD'} = 'password';
$hashsettings{'UPSDRIVER'} = 'usbhid-ups';
$hashsettings{'UPSPORT'} = 'auto';
$hashsettings{'UPSCOMMENT'} = 'my ups';

&General::getcgihash(\%hashcgi);
&General::readhash('/var/ipcop/addons/nut/settings', \%hashsettings);

&Header::showhttpheaders();

&Header::openpage('NUT', 1, '');
&Header::openbigbox('100%', 'left', '');


if ($hashcgi{'ACTION'} eq "$Lang::tr{'save'}") {
    map ($hashsettings{$_} = $hashcgi{$_}, ('MODE', 'SERVER',
        'UPSNAME', 'USERNAME', 'PASSWORD',
        'UPSDRIVER', 'UPSPORT', 'UPSCOMMENT'));

    if ($hashsettings{'MODE'} eq 'netclient') {
        unless (&General::validip($hashsettings{'SERVER'})) {
            $errormessage = $Lang::tr{'invalid ip'};
            goto ERROREXIT;
        }
    }
    elsif (($hashsettings{'MODE'} eq 'standalone') || ($hashsettings{'MODE'} eq 'standalone')) {
        if ($hashsettings{'UPSNAME'} eq '') {
            $errormessage = $Lang::tr{'name is invalid'};
            goto ERROREXIT;
        }
        # TODO: verify driver
    }

    &General::writehash('/var/ipcop/addons/nut/settings', \%hashsettings);
    &WriteConfig_nut_conf();
    system("/usr/local/bin/nutctrl --restart >/dev/null 2>&1");
}
ERROREXIT:

if ($errormessage) {
    &Header::openbox('100%', 'left', "$Lang::tr{'error messages'}:", 'error');
    print "<font class='base'>$errormessage&nbsp;</font>";
    &Header::closebox();
}

#
# Check for new version
#
&checkfornewversion($addonname, $version);
if ($onlineversion ne '') {
    &Header::openbox('100%', 'left', $Lang::tr{'info'}, 'warning');
    print <<END
<table width="100%"><tr>
<td>A new version is available: <a href="$onlinelink" target="_blank"><b>$onlineversion</b></a></td>
</tr></table>
END
    ;
    &Header::closebox();
}


$selected{'MODE'}{'none'} = '';
$selected{'MODE'}{'standalone'} = '';
$selected{'MODE'}{'netserver'} = '';
$selected{'MODE'}{'netclient'} = '';
$selected{'MODE'}{$hashsettings{'MODE'}} = "selected='selected'";

&Header::openbox('100%', 'left', 'Network UPS Tools');
my $active_mon = &General::isrunning('nut/upsmon', 'nosize');
my $active_d = &General::isrunning('nut/upsd', 'nosize');

print <<END
<form method='post' action='$ENV{'SCRIPT_NAME'}'>
<table width='100%'>
<tr>
    <td>NUT monitor:</td>
    $active_mon
    <td colspan='2'>&nbsp;</td>
END
;
print <<END
</tr><tr>
    <td>NUT daemon:</td>
    $active_d
    <td colspan='2'>&nbsp;</td>
END
unless ($hashsettings{'MODE'} eq 'netclient');
print <<END
</tr><tr>
    <td colspan='2'>&nbsp;</td>
    <td colspan='2' align='right'><b><small><a href="http://www.ban-solms.de/t/IPCop.html" target="_blank">NUT Addon $version</a></small></b></td>
</tr><tr>
    <td colspan='4'><hr /></td>
</tr><tr>
    <td width='25%' class='base'>Mode:</td>
    <td width='25%'><select name='MODE'>
        <option value='none' $selected{'MODE'}{'none'}>none (off)</option>
        <option value='standalone' $selected{'MODE'}{'standalone'} disabled='disabled'>standalone</option>
        <option value='netserver' $selected{'MODE'}{'netserver'} disabled='disabled'>netserver</option>
        <option value='netclient' $selected{'MODE'}{'netclient'}>netclient</option>
    </select></td>
    <td colspan='2'>&nbsp;</td>
</tr><tr>
    <td width='25%' class='base'>$Lang::tr{'name'}:</td>
    <td width='25%'><input type='text' name='UPSNAME' value='$hashsettings{"UPSNAME"}' size='32' /></td>
    <td colspan='2'>&nbsp;</td>
</tr><tr>
    <td width='25%' class='base'>$Lang::tr{'username'}:</td>
    <td width='25%'><input type='text' name='USERNAME' value='$hashsettings{"USERNAME"}' size='32' /></td>
    <td width='25%' class='base'>$Lang::tr{'password'}:</td>
    <td width='25%'><input type='password' name='PASSWORD' value='$hashsettings{"PASSWORD"}' size='32' /></td>
</tr><tr>
    <td colspan='4'><hr /></td>
</tr><tr>
    <td colspan='4'><b>UPS device</b></td>
</tr><tr>
    <td width='25%' class='base'>$Lang::tr{'driver'}:</td>
    <td width='25%'><input type='text' name='UPSDRIVER' value='$hashsettings{"UPSDRIVER"}' size='32' disabled='disabled' /></td>
    <td width='25%' class='base'>$Lang::tr{'port'}:</td>
    <td width='25%'><input type='text' name='UPSPORT' value='$hashsettings{"UPSPORT"}' size='32' disabled='disabled' /></td>
</tr><tr>
    <td colspan='4'><hr /></td>
</tr><tr>
    <td colspan='4'><b>Netclient mode</b></td>
</tr><tr>
    <td width='25%'>NUT server:&nbsp;<img src='/blob.gif' alt='*' /></td>
    <td width='25%'><input type='text' name='SERVER' value='$hashsettings{"SERVER"}' size='32' /></td>
    <td colspan='2'>&nbsp;</td>
</tr>
</table>
<hr />
<table width='100%'>
<tr>
    <td class='comment1button'><img src='/blob.gif' align='top' alt='*' />&nbsp;$Lang::tr{'this field may be blank'}</td>
    <td class='button1button'>
        <input type='submit' name='ACTION' value='$Lang::tr{'save'}' />
    </td>
    <td class='onlinehelp'>
        <a href='http://www.ban-solms.de/t/IPCop-nut.html' target='_blank'>
        <img src='/images/web-support.png' alt='Online help' title='Online help' /></a>
    </td>
</tr>
</table>
</form>
END
;

&Header::closebox();
&Header::closebigbox();
&Header::closepage();


sub checkfornewversion
{
    my $addon = shift;
    my $version = shift;
    $onlineversion = '';

    if (-e $flagdonotcheck) {
        return;
    }

    # only check if we are online and last check was some time ago
    if ((! -e '/var/ipcop/red/active') || (-e $timestamplastcheck) && (int(-M $timestamplastcheck) < 5)) {
        return;
    }

#workaround to suppress a warning when a variable is used only once
    my @dummy = ( $General::version );
    undef (@dummy);

    my $ua = LWP::UserAgent->new;
    $ua->timeout(120);
    $ua->agent("Mozilla/4.0 (compatible; IPCop $General::version; $version)");
    my $content = $ua->get("http://www.ban-solms.de/version/$addon");

    if ($content->is_success) {
        # compare the versions, format is v1.2.3
        $content->content =~ /v(\d+).(\d+).(\d+)/;
        my $ver1 = $1;
        my $ver2 = $2;
        my $ver3 = $3;

        $version =~ /v(\d+).(\d+).(\d+)/;

        if ( ($ver1 > $1) || (($ver1 == $1) && ($ver2 > $2)) || (($ver1 == $1) && ($ver2 == $2) && ($ver3 > $3)) ) {
            $onlineversion = "v$ver1.$ver2.$ver3";

            $content->content =~ /http(.*)/;
            $onlinelink = "http$1";
        }
        else {
            # no news, recheck in a couple of days
            system("touch $timestamplastcheck");
        }
    }
}


sub WriteConfig_nut_conf
{
    my $std_header = "#\n# config-file for NUT written by IPCop NUT Addon\n#";
    my $server = 'localhost';
    my $masterslave = 'master';
    $server = $hashsettings{'SERVER'} if ($hashsettings{'MODE'} eq 'netclient');
    $masterslave = 'slave' if ($hashsettings{'MODE'} eq 'netclient');

########################################
#
# nut.conf
#
########################################
    open (CONFIGFILE, ">/var/ipcop/addons/nut/nut.conf");
    print CONFIGFILE <<END
$std_header

MODE=$hashsettings{'MODE'}
END
    ;
    close CONFIGFILE;

########################################
#
# ups.conf
#
########################################
    open (CONFIGFILE, ">/var/ipcop/addons/nut/ups.conf");
    print CONFIGFILE <<END
$std_header

[$hashsettings{'UPSNAME'}]
    driver = $hashsettings{'UPSDRIVER'}
    port = $hashsettings{'UPSPORT'}
END
    ;
    close CONFIGFILE;


    open (CONFIGFILE, ">/var/ipcop/addons/nut/upsd.conf");
    print CONFIGFILE <<END
$std_header

LISTEN 127.0.0.1 3493

END
    ;
    close CONFIGFILE;

########################################
#
# upsd.users
#
########################################
    open (CONFIGFILE, ">/var/ipcop/addons/nut/upsd.users");
    print CONFIGFILE <<END
$std_header

[$hashsettings{'USERNAME'}]
    password = $hashsettings{'PASSWORD'}
    upsmon slave
END
    ;
    close CONFIGFILE;

########################################
#
# upsmon.conf
#
########################################
    open (CONFIGFILE, ">/var/ipcop/addons/nut/upsmon.conf");
    print CONFIGFILE <<END
$std_header

MONITOR $hashsettings{'UPSNAME'}\@$server 1 $hashsettings{'USERNAME'} $hashsettings{'PASSWORD'} $masterslave
MINSUPPLIES 1
SHUTDOWNCMD "/usr/local/bin/ipcopreboot --down UPS shutdown"

POLLFREQ 15
POLLFREQALERT 5
HOSTSYNC 15
DEADTIME 60

POWERDOWNFLAG /var/ipcop/addons/nut/killpower

NOTIFYFLAG ONLINE     SYSLOG
NOTIFYFLAG ONBATT     SYSLOG
NOTIFYFLAG LOWBATT    SYSLOG
NOTIFYFLAG FSD        SYSLOG
NOTIFYFLAG COMMOK     SYSLOG
NOTIFYFLAG COMMBAD    SYSLOG
NOTIFYFLAG SHUTDOWN   SYSLOG
NOTIFYFLAG REPLBATT   SYSLOG
NOTIFYFLAG NOCOMM     SYSLOG
NOTIFYFLAG NOPARENT   SYSLOG

FINALDELAY 5

END
    ;
    close CONFIGFILE;

    chmod(0640, '/var/ipcop/addons/nut/upsd.conf', '/var/ipcop/addons/nut/upsd.users', '/var/ipcop/addons/nut/upsmon.conf');
}
